import pytest
from omnisci.cursor import Cursor, _bind_parameters
from omnisci import connect


@pytest.fixture
def mock_connection(mock_client):
    """Connection with mocked transport layer, and

    - username='user'
    - password='password'
    - host='localhost'
    - dbname='dbname'
    """
    return connect(
        user='user', password='password', host='localhost', dbname='dbname'
    )


class TestCursor:
    def test_empty_iterable(self):
        c = Cursor(None)
        result = list(c)
        assert result == []

    def test_escape_basic(self):
        query = "select * from foo where bar > :baz"
        result = str(_bind_parameters(query, {"baz": 10}))
        expected = 'select * from foo where bar > 10'
        assert result == expected

    def test_escape_malicious(self):
        query = "select * from foo where bar > :baz"
        result = str(_bind_parameters(query, {"baz": '1; drop table foo'}))
        # note the inner quotes
        expected = "select * from foo where bar > '1; drop table foo'"
        assert result == expected

    def test_arraysize(self):
        c = Cursor(None)
        assert c.arraysize == 1
        c.arraysize = 10
        assert c.arraysize == 10

        with pytest.raises(TypeError):
            c.arraysize = 'a'



from omnisci.cursor import make_row_results_set
from omnisci.thrift.ttypes import (
    TRowSet,
    TColumnType,
    TColumn,
    TColumnData,
    TQueryResult,
)
from omnisci.common.ttypes import TTypeInfo


class TestRowResults:
    def test_nulls_handled(self):

        rs = TQueryResult(
            TRowSet(
                row_desc=[
                    TColumnType(
                        col_name='a', col_type=TTypeInfo(type=0, nullable=True)
                    ),
                    TColumnType(
                        col_name='b', col_type=TTypeInfo(type=1, nullable=True)
                    ),
                    TColumnType(
                        col_name='c', col_type=TTypeInfo(type=2, nullable=True)
                    ),
                    TColumnType(
                        col_name='d', col_type=TTypeInfo(type=3, nullable=True)
                    ),
                    TColumnType(
                        col_name='e', col_type=TTypeInfo(type=4, nullable=True)
                    ),
                    TColumnType(
                        col_name='f', col_type=TTypeInfo(type=5, nullable=True)
                    ),
                    TColumnType(
                        col_name='g', col_type=TTypeInfo(type=6, nullable=True)
                    ),
                    TColumnType(
                        col_name='h', col_type=TTypeInfo(type=7, nullable=True)
                    ),
                    TColumnType(
                        col_name='i', col_type=TTypeInfo(type=8, nullable=True)
                    ),
                    TColumnType(
                        col_name='j', col_type=TTypeInfo(type=9, nullable=True)
                    ),
                    TColumnType(
                        col_name='k',
                        col_type=TTypeInfo(type=10, nullable=True),
                    ),
                ],
                rows=[],
                columns=[
                    TColumn(
                        data=TColumnData(int_col=[-2147483648]), nulls=[True]
                    ),
                    TColumn(
                        data=TColumnData(int_col=[-2147483648]), nulls=[True]
                    ),
                    TColumn(
                        data=TColumnData(int_col=[-2147483648]), nulls=[True]
                    ),
                    TColumn(
                        data=TColumnData(real_col=[-2147483648]), nulls=[True]
                    ),  # noqa
                    TColumn(
                        data=TColumnData(real_col=[-2147483648]), nulls=[True]
                    ),  # noqa
                    TColumn(
                        data=TColumnData(real_col=[-2147483648]), nulls=[True]
                    ),  # noqa
                    TColumn(
                        data=TColumnData(str_col=[-2147483648]), nulls=[True]
                    ),
                    TColumn(
                        data=TColumnData(int_col=[-2147483648]), nulls=[True]
                    ),
                    TColumn(
                        data=TColumnData(int_col=[-2147483648]), nulls=[True]
                    ),
                    TColumn(
                        data=TColumnData(int_col=[-2147483648]), nulls=[True]
                    ),
                    TColumn(
                        data=TColumnData(int_col=[-2147483648]), nulls=[True]
                    ),
                ],
                is_columnar=True,
            )
        )

        result = list(make_row_results_set(rs))
        assert result == [(None,) * 11]



import os
import pytest
from omnisci.thrift.ttypes import TColumnType
from omnisci.common.ttypes import TTypeInfo
from omnisci import OperationalError, connect
from omnisci.connection import _parse_uri, ConnectionInfo
from omnisci.exceptions import Error
from omnisci._parsers import ColumnDetails, _extract_column_details

omniscihost = os.environ.get('OMNISCI_HOST', 'localhost')

@pytest.mark.usefixtures("omnisci_server")
class TestConnect:
    def test_host_specified(self):
        with pytest.raises(TypeError):
            connect(user='foo')

    def test_raises_right_exception(self):
        with pytest.raises(OperationalError):
            connect(host=omniscihost, protocol='binary', port=1234)

    def test_close(self):
        conn = connect(
            user='admin',
            password='HyperInteractive',
            host=omniscihost,
            dbname='omnisci',
        )
        assert conn.closed == 0
        conn.close()
        assert conn.closed == 1

    def test_commit_noop(self, con):
        result = con.commit()  # it worked
        assert result is None

    def test_bad_protocol(self, mock_client):
        with pytest.raises(ValueError) as m:
            connect(
                user='user',
                host=omniscihost,
                dbname='dbname',
                protocol='fake-proto',
            )
        assert m.match('fake-proto')

    def test_session_logon_success(self):
        conn = connect(
            user='admin',
            password='HyperInteractive',
            host=omniscihost,
            dbname='omnisci',
        )
        sessionid = conn._session
        connnew = connect(sessionid=sessionid, host=omniscihost)
        assert connnew._session == sessionid

    def test_session_logon_failure(self):
        sessionid = 'ILoveDancingOnTables'
        with pytest.raises(Error):
            connect(sessionid=sessionid, host=omniscihost, protocol='binary')

    def test_bad_binary_encryption_params(self):
        with pytest.raises(TypeError):
            connect(
                user='admin',
                host=omniscihost,
                dbname='omnisci',
                protocol='http',
                validate=False,
            )


class TestURI:
    def test_parse_uri(self):
        uri = (
            'omnisci://admin:HyperInteractive@{0}:6274/omnisci?'
            'protocol=binary'.format(omniscihost)
        )
        result = _parse_uri(uri)
        expected = ConnectionInfo(
            "admin",
            "HyperInteractive",
            omniscihost,
            6274,
            "omnisci",
            "binary",
            None,
            None,
        )
        assert result == expected

    def test_both_raises(self):
        uri = (
            'omnisci://admin:HyperInteractive@{0}:6274/omnisci?'
            'protocol=binary'.format(omniscihost)
        )
        with pytest.raises(TypeError):
            connect(uri=uri, user='my user')


class TestExtras:
    def test_extract_row_details(self):
        data = [
            TColumnType(
                col_name='date_',
                col_type=TTypeInfo(
                    type=6,
                    encoding=4,
                    nullable=True,
                    is_array=False,
                    precision=0,
                    scale=0,
                    comp_param=32,
                ),
                is_reserved_keyword=False,
                src_name='',
            ),
            TColumnType(
                col_name='trans',
                col_type=TTypeInfo(
                    type=6,
                    encoding=4,
                    nullable=True,
                    is_array=False,
                    precision=0,
                    scale=0,
                    comp_param=32,
                ),
                is_reserved_keyword=False,
                src_name='',
            ),
            TColumnType(
                col_name='symbol',
                col_type=TTypeInfo(
                    type=6,
                    encoding=4,
                    nullable=True,
                    is_array=False,
                    precision=0,
                    scale=0,
                    comp_param=32,
                ),
                is_reserved_keyword=False,
                src_name='',
            ),
            TColumnType(
                col_name='qty',
                col_type=TTypeInfo(
                    type=1,
                    encoding=0,
                    nullable=True,
                    is_array=False,
                    precision=0,
                    scale=0,
                    comp_param=0,
                ),
                is_reserved_keyword=False,
                src_name='',
            ),
            TColumnType(
                col_name='price',
                col_type=TTypeInfo(
                    type=3,
                    encoding=0,
                    nullable=True,
                    is_array=False,
                    precision=0,
                    scale=0,
                    comp_param=0,
                ),
                is_reserved_keyword=False,
                src_name='',
            ),
            TColumnType(
                col_name='vol',
                col_type=TTypeInfo(
                    type=3,
                    encoding=0,
                    nullable=True,
                    is_array=False,
                    precision=0,
                    scale=0,
                    comp_param=0,
                ),
                is_reserved_keyword=False,
                src_name='',
            ),
        ]
        result = _extract_column_details(data)

        expected = [
            ColumnDetails(
                name='date_',
                type='STR',
                nullable=True,
                precision=0,
                scale=0,
                comp_param=32,
                encoding='DICT',
                is_array=False,
            ),
            ColumnDetails(
                name='trans',
                type='STR',
                nullable=True,
                precision=0,
                scale=0,
                comp_param=32,
                encoding='DICT',
                is_array=False,
            ),
            ColumnDetails(
                name='symbol',
                type='STR',
                nullable=True,
                precision=0,
                scale=0,
                comp_param=32,
                encoding='DICT',
                is_array=False,
            ),
            ColumnDetails(
                name='qty',
                type='INT',
                nullable=True,
                precision=0,
                scale=0,
                comp_param=0,
                encoding='NONE',
                is_array=False,
            ),
            ColumnDetails(
                name='price',
                type='FLOAT',
                nullable=True,
                precision=0,
                scale=0,
                comp_param=0,
                encoding='NONE',
                is_array=False,
            ),
            ColumnDetails(
                name='vol',
                type='FLOAT',
                nullable=True,
                precision=0,
                scale=0,
                comp_param=0,
                encoding='NONE',
                is_array=False,
            ),
        ]
        assert result == expected
